@@ -2,90 +2,106 @@
 # Copyright (C) 2005 - 2008 by Torsten Knorr
 # create-soft@tiscali.de
 # All rights reserved!
-#------------------------------------------------
-package Net::UploadMirror;
-#------------------------------------------------
+#-------------------------------------------------
  use strict;
+#------------------------------------------------
+ package Net::UploadMirror::FileName;
+ use Storable;
+ sub TIESCALAR{ my ($class, $obj) = @_; return(bless(\$obj, $class || ref($class))); }
+ sub STORE 
+ 	{
+ 	if(-f $_[1])
+ 		{
+ 		${$_[0]}->{_last_modified} = retrieve($_[1]);
+ 		}
+ 	else
+ 		{
+ 		${$_[0]}->{_last_modified} = {};
+ 		store(${$_[0]}->{_last_modified}, $_[1]);
+ 		warn("\nno information of the files last modified times\n");
+ 		}
+ 	}
+ sub FETCH { return(${$_[0]}->{_filename}); }
+#-------------------------------------------------
+ package Net::UploadMirror;
+#-------------------------------------------------
  use Net::MirrorDir;
  use Storable;
  use File::Basename;
  use vars '$AUTOLOAD';
 #------------------------------------------------
  @Net::UploadMirror::ISA = qw(Net::MirrorDir);
- $Net::UploadMirror::VERSION = '0.08';
+ $Net::UploadMirror::VERSION = '0.09';
 #-------------------------------------------------
  sub _Init
  	{
  	my ($self, %arg) = @_;
- 	$self->{_file_name} = $arg{file_name} || "lastmodified_local";
- 	if(-f $self->{_file_name})
- 		{
- 		$self->{_last_modified} = retrieve($self->{_file_name});
- 		}
- 	else
- 		{
- 		$self->{_last_modified} = {};
- 		store($self->{_last_modified}, $self->{_file_name});
- 		warn("\nno information of the files last modified times\n");
- 		return(0);
- 		}
+	tie($self->{_filename}, "Net::UploadMirror::FileName", $self);
+ 	$self->{_filename}	= $arg{filename}	|| "lastmodified_local";
+ 	$self->{_delete}	= $arg{delete}	|| "disabled";
  	return(1);
  	}
 #-------------------------------------------------
  sub Update
  	{
  	my ($self) = @_;
- 	$self->Connect() if(!(defined($self->{_connection})));
  	my ($ref_h_local_files, $ref_h_local_dirs) = $self->ReadLocalDir();
  	if($self->{_debug})
  		{
  		print("local files : $_\n") for(sort keys %{$ref_h_local_files});
  		print("local dirs : $_\n") for(sort keys %{$ref_h_local_dirs});
  		}
+ 	my $ref_a_modified_local_files = $self->CheckIfModified($ref_h_local_files);
+ 	if($self->{_debug})
+ 		{
+ 		print("modified files : $_\n") for(@{$ref_a_modified_local_files});
+ 		}
+ 	return(0) if(!$self->Connect());
+	$self->StoreFiles($ref_a_modified_local_files);
  	my ($ref_h_remote_files, $ref_h_remote_dirs) = $self->ReadRemoteDir();
  	if($self->{_debug})
  		{
  		print("remote files : $_\n") for(sort keys %{$ref_h_remote_files});
  		print("remote dirs : $_\n") for(sort keys %{$ref_h_remote_dirs});
  		}
- 	my $ref_a_new_local_files = $self->LocalNotInRemote(
- 		$ref_h_local_files, $ref_h_remote_files);
+	my $ref_a_new_local_files = $self->LocalNotInRemote(
+ 		$ref_h_local_files,
+		$ref_h_remote_files
+		);
  	if($self->{_debug})
  		{
- 		print("new files : $_\n") for(@{$ref_a_new_local_files});
+ 		print("local files not in remote: $_\n") for(@{$ref_a_new_local_files});
  		}
- 	$self->StoreFiles($ref_a_new_local_files) if(@{$ref_a_new_local_files});
+ 	$self->StoreFiles($ref_a_new_local_files);
  	my $ref_a_new_local_dirs = $self->LocalNotInRemote(
- 		$ref_h_local_dirs, $ref_h_remote_dirs);
+ 		$ref_h_local_dirs,
+ 		$ref_h_remote_dirs
+ 		);
  	if($self->{_debug})
  		{
- 		print("new dirs : $_\n") for(@{$ref_a_new_local_dirs});
+ 		print("local directories not in remote : $_\n") for(@{$ref_a_new_local_dirs});
  		}
- 	$self->MakeDirs($ref_a_new_local_dirs) if(@{$ref_a_new_local_dirs});
+ 	$self->MakeDirs($ref_a_new_local_dirs);
  	if($self->{_delete} eq "enable")
  		{
  		my $ref_a_deleted_local_files = $self->RemoteNotInLocal(
- 			$ref_h_local_files, $ref_h_remote_files);
+ 			$ref_h_local_files,
+ 			$ref_h_remote_files
+ 			);
  		if($self->{_debug})
  			{
- 			print("deleted files : $_\n") for(@{$ref_a_deleted_local_files});
+ 			print("remote files not in local : $_\n") for(@{$ref_a_deleted_local_files});
  			}
-  		$self->DeleteFiles($ref_a_deleted_local_files) if(@{$ref_a_deleted_local_files});
+  		$self->DeleteFiles($ref_a_deleted_local_files);
  		my $ref_a_deleted_local_dirs = $self->RemoteNotInLocal(
- 			$ref_h_local_dirs, $ref_h_remote_dirs);
+ 			$ref_h_local_dirs,
+ 			$ref_h_remote_dirs);
  		if($self->{_debug})
  			{
- 			print("deleted dirs : $_\n") for(@{$ref_a_deleted_local_dirs});
+ 			print("remote directories not in local : $_\n") for(@{$ref_a_deleted_local_dirs});
  			}
- 		$self->RemoveDirs($ref_a_deleted_local_dirs) if(@{$ref_a_deleted_local_dirs});
+ 		$self->RemoveDirs($ref_a_deleted_local_dirs);
  		}
- 	delete($ref_h_local_files->{$_}) for(@{$ref_a_new_local_files});
- 	my $ref_a_modified_local_files = $self->CheckIfModified($ref_h_local_files);
- 	if($self->{_debug})
- 		{
- 		print("modified files : $_\n") for(@{$ref_a_modified_local_files});
- 		}
- 	$self->StoreFiles($ref_a_modified_local_files) if(@{$ref_a_modified_local_files});
  	$self->Quit();
  	return(1);
  	}
@@ -93,7 +109,7 @@ package Net::UploadMirror;
  sub CheckIfModified
  	{
  	my ($self, $ref_h_local_files) = @_;
- 	my (@modified_files, $ref_h_last_modified);
+ 	my @modified_files;
  	for(keys(%{$ref_h_local_files}))
  		{
  		next if((-d $_) || !(-f $_));
@@ -109,14 +125,14 @@ package Net::UploadMirror;
 sub StoreFiles
  	{
  	my ($self, $ref_a_files) = @_;
- 	my ($l_path, $r_path, $ref_h_last_modified);
- 	return(0) if(!(defined($self->{_connection})));
+ 	return(0) if(!$self->IsConnection() or !@{$ref_a_files});
+ 	my ($l_path, $r_path);
  	for(@{$ref_a_files})
  		{
  		if(-f $_)
  			{
  			$r_path = $l_path = $_;
- 			$r_path =~ s!^$self->{_localdir}!$self->{_remotedir}!;
+ 			$r_path =~ s!$self->{_regex_localdir}!$self->{_remotedir}!;
  			my ($r_name, $r_dir, $r_sufix) = fileparse($r_path);
  			if(!($self->{_connection}->cwd($r_dir)))
  				{
@@ -129,31 +145,29 @@ sub StoreFiles
  			}
  		else
  			{
- 			warn("error in StoreFiles() : $l_path is not a file\n");
+ 			warn("error in StoreFiles() : $_ is not a file\n");
  			}
  		}
- 	store($self->{_last_modified}, $self->{_file_name});
+ 	store($self->{_last_modified}, $self->{_filename});
  	return(1);
  	}
 #-------------------------------------------------
  sub MakeDirs
  	{
  	my ($self, $ref_a_dirs) = @_;
- 	my ($l_dir, $r_dir);
- 	return(0) if(!(defined($self->{_connection})));
- 	for(@{$ref_a_dirs})
+ 	return(0) if(!$self->IsConnection() or !@{$ref_a_dirs});
+ 	for my $r_dir (@{$ref_a_dirs})
  		{
- 		if(-d $_)
+ 		if(-d $r_dir)
  			{
- 			$l_dir = $r_dir = $_;
- 			$r_dir =~ s!^$self->{_localdir}!$self->{_remotedir}!;
+ 			$r_dir =~ s!$self->{_regex_localdir}!$self->{_remotedir}!;
  			next if($self->{_connection}->cwd($r_dir));
  			$self->{_connection}->cwd();
  			$self->{_connection}->mkdir($r_dir, 1) ;
  			}
  		else
  			{
-			warn("error in MakeDirs() : $l_dir is not a directory\n");
+			warn("error in MakeDirs() : $r_dir is not a directory\n");
  			}
  		}
  	return(1);
@@ -162,14 +176,11 @@ sub StoreFiles
  sub DeleteFiles
  	{
  	my ($self, $ref_a_files) = @_;
- 	my ($l_path, $ref_h_last_modified);
- 	return(0) if(!($self->{_delete} eq "enable"));
- 	return(0) if(!(defined($self->{_connection})));
- 	for(@{$ref_a_files})
+	return(0) if(($self->{_delete} ne "enable") or !$self->IsConnection() or !@{$ref_a_files});
+ 	for my $l_path (@{$ref_a_files})
  		{
- 		$l_path = $_; 
- 		$l_path =~ s!^$self->{_remotedir}!$self->{_localdir}!;
- 		$self->{_connection}->delete($_);
+		$self->{_connection}->delete($l_path);
+ 		$l_path =~ s!$self->{_regex_remotedir}!$self->{_localdir}!;
  		delete($self->{_last_modified}{$l_path}) if(defined($self->{_last_modified}{$l_path}));
  		}
  	store($self->{_last_modified}, $self->{_file_name});
@@ -179,8 +190,7 @@ sub StoreFiles
  sub RemoveDirs
  	{
  	my ($self, $ref_a_dirs) = @_;
- 	return(0) if(!(defined($self->{_connection})));
- 	return(0) if(!($self->{_delete} eq "enable"));
+	return(0) if(($self->{_delete} ne "enable") or !$self->IsConnection() or !@{$ref_a_dirs});
  	$self->{_connection}->rmdir($_, 1) for(@{$ref_a_dirs});
  	return(1);
  	}
@@ -224,7 +234,7 @@ Net::UploadMirror - Perl extension for mirroring a local directory via FTP to th
 # or you can use regular expressions
 # 	exclusinos	=> [qr/SYSTEM/i, $regex]
 # 	subset		=> {qr/(?i:HOME)(?i:PAGE)?/, $regex]
- 	file_name	=> "modified_times"
+ 	filename	=> "modified_times"
  	);
  $um->Update();
 
@@ -236,35 +246,36 @@ worked on locally. It is not developt for mirroring large archivs.
 But there are not in principle any limits.
 
 =head1 Constructor and Initialization
-=item (object) new (options)
+=item (object)new(options)
  Net::UploadMirror is a derived class from Net::MirrorDir.
  For detailed information about constructor or options
  read the documentation of Net::MirrorDir.
 
 =head2 methods
 
-=item (1) _Init(%arg)
+=item (1)_Init(%arg)
  This function is called by the constructor.
  You do not need to call this function by yourself.
 
-=item (1) Update (void)
+=item (1|0)Update(void)
  Call this function for mirroring automatically, recommended!!!
 
-=item (ref_hash_modified_files) CheckIfModified (ref_list_local_files)
- Takes a hashreference of local files to compare the last modification stored in a file
- "lastmodified_local" while uploading. Returns a reference of a list.
+=item (ref_hash_modified_files)CheckIfModified(ref_list_local_files)
+ Takes a hashreference of local filenames to compare the last modification time,
+ which is stored in a file, named by the attribute "filename", while uploading. 
+ Returns a reference of a list.
 
-=item (1) StoreFiles (ref_list_paths)
- Takes a listreference of local-paths to upload the files via FTP.
+=item (1|0)StoreFiles(ref_array_paths)
+ Takes a arrayreference of local-paths to upload the files via FTP.
 
-=item (1) MakeDirs (ref_list_paths)
- Takes a listreference of directorys to make in the remote location.
+=item (1|0)MakeDirs(ref_array_paths)
+ Takes a arrayreference of directories to make in the remote location.
 
-=item (1) DeleteFiles (ref_list_paths)
- Takes a listreference of files to delete in the remote location.
+=item (1|0)DeleteFiles(ref_array_paths)
+ Takes a arrayreference of files to delete in the remote location.
 
-=item (1) RemoveDirs (ref_list_paths)
- Takes a listreference of directories to remove in the remote location.
+=item (1|0)RemoveDirs(ref_array_paths)
+ Takes a arrayreference of directories to remove in the remote location.
 
 =head2 optional optiones
 
@@ -272,6 +283,10 @@ But there are not in principle any limits.
  The name of the file in which the last modified times will be stored.
  default = "lastmodified_local"
 
+=item delete
+ When directories or files are to be deleted = "enable"
+ default = "disabled"
+ 
 =head2 EXPORT
 
 None by default.
@@ -291,19 +306,19 @@ File::Basename
 
 =head1 BUGS
 
-Maybe you'll find some. Let me know.
+ Maybe you'll find some. Let me know.
 
 =head1 REPORTING BUGS
 
-When reporting bugs/problems please include as much information as possible.
+ When reporting bugs/problems please include as much information as possible.
 
 =head1 AUTHOR
 
-Torsten Knorr, E<lt>create-soft@tiscali.deE<gt>
+ Torsten Knorr, E<lt>create-soft@tiscali.deE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005 - 2008 by Torsten Knorr
+ Copyright (C) 2005 - 2008 by Torsten Knorr
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.9.2 or,
@@ -313,5 +328,3 @@ at your option, any later version of Perl 5 you may have available.
 =cut
 
 
-
-
